	; CVBasic compiler v0.5.1 Jun/23/2024
	; Command: cvbasic invader.bas invader.asm 
	; Created: Mon Jul 08 16:26:14 2024

COLECO:	equ 1
SG1000:	equ 0
MSX:	equ 0
SGM:	equ 0
SVI:	equ 0

CVBASIC_MUSIC_PLAYER:	equ 0
CVBASIC_COMPRESSION:	equ 0
CVBASIC_BANK_SWITCHING:	equ 0

BASE_RAM:	equ $7000	; Base of RAM
STACK:	equ $7400	; Base stack pointer
VDP:	equ $be	; VDP port (write)
VDPR:	equ $be	; VDP port (read)

	;
	; CVBasic prologue (BASIC compiler for Colecovision)
	;
	; by Oscar Toledo G.
	; https://nanochess.org/
	;
	; Creation date: Feb/27/2024.
	; Revision date: Feb/29/2024. Turns off sound. Now it reads the controllers.
	;                             Added 16-bit multiply, division, modulo, and abs.
	;                             Added random generator. Added sound routines.
	; Revision date: Mar/03/2024. Removed fname directive to use gasm80.
	; Revision date: Mar/05/2024. Added support for Sega SG1000.
	; Revision date: Mar/06/2024. Added ENASCR, DISSCR, and CPYBLK.
	; Revision date: Mar/08/2024. Added modes 0, 1 and 2.
	; Revision date: Mar/12/2024. Added support for MSX.
	; Revision date: Mar/14/2024. Added _sgn16.
	; Revision date: Mar/15/2024. Added upper 16k enable for MSX.
	; Revision date: Apr/11/2024. Added support for formatting numbers. Added
	;                             support for Super Game Module.
	; Revision date: Apr/13/2024. Saved bytes in SG-1000 ROMs. Faster LDIRVM.
	;                             Shorter mode setting subroutines.
	; Revision date: Apr/26/2024. Interruption handler saves current bank.
	; Revision date: Apr/27/2024. Music player now supports bank switching.
	; Revision date: May/17/2024. Added delay for SG1000 and SC3000 controller
	;                             support with keyboard (code by SiRioKD)
	; Revision date: Jun/04/2024. SGM supported deleted NTSC flag.
	; Revision date: Jun/07/2024. Keys 0-9, = and - emulate keypad in MSX.
	; Revision date: Jun/17/2024. Added SVI-328 support.
	;

JOYSEL:	equ $c0
KEYSEL:	equ $80

PSG:    equ $ff-$80*SG1000
JOY1:   equ $fc-$20*SG1000
JOY2:   equ $ff-$22*SG1000

    if COLECO
	org $8000
	db $55,$aa
	dw 0
	dw 0
	dw 0
	dw 0
	dw START

	jp 0	; rst $08
	jp 0	; rst $10
	jp 0	; rst $18
	jp 0	; rst $20
	jp 0	; rst $28
	jp 0	; rst $30
	jp 0	; rst $38

	jp nmi_handler
    endif
    if SG1000+SVI
	org $0000
	di
	ld sp,STACK
	jp START
	db $ff
	jp 0
	db $ff,$ff,$ff,$ff,$ff
	jp 0
	db $ff,$ff,$ff,$ff,$ff
	jp 0
	db $ff,$ff,$ff,$ff,$ff
	jp 0
	db $ff,$ff,$ff,$ff,$ff
	jp 0
	db $ff,$ff,$ff,$ff,$ff
	jp 0
	db $ff,$ff,$ff,$ff,$ff
	jp nmi_handler	; It should be called int_handler.
    endif
    if MSX
	ORG $4000
	db "AB"
	dw START
	dw $0000
	dw $0000
	dw $0000
	dw $0000

WRTPSG:	equ $0093
RDPSG:	equ $0096

    endif
    if SVI
WRTPSG:	
	out ($88),a
	push af
	ld a,e
	out ($8c),a
	pop af
	ret

RDPSG:
	out ($88),a
	push af
	pop af
	in a,($90)
	ret
    endif

WRTVDP:
	ld a,b
	out (VDP+1),a
	ld a,c
	or $80
	out (VDP+1),a
	ret

SETWRT:
	ld a,l
	out (VDP+1),a
	ld a,h
	or $40
	out (VDP+1),a
	ret

SETRD:
	ld a,l
	out (VDP+1),a
	ld a,h
        and $3f
	out (VDP+1),a
	ret

WRTVRM:
	push af
	call SETWRT
	pop af
	out (VDP),a
	ret

    if SG1000
	db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

	; Located at $0066
	ei		; NMI handler (pause button)
	retn
    endif

RDVRM:
        push af
        call SETRD
        pop af
        ex (sp),hl
        ex (sp),hl
        in a,(VDPR)
        ret

FILVRM:
	push af
	call SETWRT
	pop af
	dec bc		; T-states (normal / M1)
.1:	out (VDP),a	; 11 12
	dec bc		;  6  7
	bit 7,b		;  8 10
	jp z,.1		; 10 11
			; -- --
			; 35 40
	ret

LDIRVM:
        EX DE,HL
        CALL SETWRT
        EX DE,HL
        DEC BC
        INC C
        LD A,B
        LD B,C
        INC A
        LD C,VDP
.1:
    if SG1000
	NOP	; SG1000 is faster (reported by SiRioKD)
    endif
	OUTI
        JP NZ,.1
        DEC A
        JP NZ,.1
        RET

LDIRVM3:
	call .1
	call .1
.1:	push hl
	push de
	push bc
	call LDIRVM
	pop bc
	pop de
	ld a,d
	add a,8
	ld d,a
	pop hl
	ret

DISSCR:
	call nmi_off
	ld bc,$a201
	call WRTVDP
	jp nmi_on

ENASCR:
	call nmi_off
	ld bc,$e201
	call WRTVDP
	jp nmi_on

CPYBLK:
	pop hl
	ex af,af'
	pop af
	ld b,a
	pop af
	ld c,a
	pop de
	ex (sp),hl
	call nmi_off
.1:	push bc
	push hl
	push de
	ld b,0
	call LDIRVM
	pop hl
	ld bc,$0020
	add hl,bc
	ex de,hl
	pop hl
	ex af,af'
	ld c,a
	ld b,0
	add hl,bc
	ex af,af'
	pop bc
	djnz .1
	jp nmi_on
	
nmi_off:
    if COLECO
	push hl
	ld hl,mode
	set 0,(hl)
	pop hl
    endif
    if SG1000+MSX+SVI
        di
    endif
	ret

nmi_on:
    if COLECO
	push af
	push hl
	ld hl,mode
	res 0,(hl)
	nop
	bit 1,(hl)
	jp nz,nmi_handler.0
	pop hl
	pop af
    endif
    if SG1000+MSX+SVI
        ei
    endif
	ret

    if COLECO
keypad_table:
        db $0f,$08,$04,$05,$0c,$07,$0a,$02
        db $0d,$0b,$00,$09,$03,$01,$06,$0f
    endif

cls:
	ld hl,$1800
	ld (cursor),hl
	ld bc,$0300
	ld a,$20
	call nmi_off
	call FILVRM
	jp nmi_on

print_string:
	ld c,a
	ld b,0
	ld de,(cursor)
	ld a,d
	and $03
	or $18
	ld d,a
	push de
	push bc
	call nmi_off
	call LDIRVM
	call nmi_on
	pop bc
	pop hl
	add hl,bc
	ld (cursor),hl
	ret

print_number:
	ld b,0
	call nmi_off
print_number5:
	ld de,10000
	call print_digit
print_number4:
	ld de,1000
	call print_digit
print_number3:
	ld de,100
	call print_digit
print_number2:
	ld de,10
	call print_digit
print_number1:
	ld de,1
	ld b,e
	call print_digit
	jp nmi_on

print_digit:
	ld a,$2f
	or a
.2:	inc a
	sbc hl,de
	jp nc,.2
	add hl,de
	cp $30
	jr nz,.3
	ld a,b
	or a
	ret z
	dec a
	jr z,.4
	ld a,c
	jr .5	
.4:
	ld a,$30
.3:	ld b,1
.5:	push hl
	ld hl,(cursor)
	ex af,af'
	ld a,h
	and $03
	or $18
	ld h,a
	ex af,af'
	call WRTVRM
	inc hl
	ld (cursor),hl
	pop hl
	ret

define_sprite:
	ex de,hl
	ld l,a
	ld h,0
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	add hl,hl	; x16
	add hl,hl	; x32
	ld c,l
	ld b,h
	pop af
	pop hl
	push af
	add hl,hl	; x2
	add hl,hl	; x4
	ld h,$07
	add hl,hl	; x8
	add hl,hl	; x16
	add hl,hl	; x32
	ex de,hl
	call nmi_off
	call LDIRVM
	jp nmi_on
	
define_char:
	ex de,hl
	ld l,a
	ld h,0
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	ld c,l
	ld b,h
	pop af
	pop hl
	push af
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	ex de,hl
	call nmi_off
	ld a,(mode)
	and 4
	jr nz,.1
	call LDIRVM3
	jp nmi_on
	
.1:	call LDIRVM
	jp nmi_on

define_color:
	ex de,hl
	ld l,a
	ld h,0
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	ld c,l
	ld b,h
	pop af
	pop hl
	push af
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	ex de,hl
	set 5,d
	call nmi_off
	call LDIRVM3
	jp nmi_on
	
update_sprite:
	pop bc
	ld (sprite_data+3),a
	pop af
	ld (sprite_data+2),a
	pop af
	ld (sprite_data+1),a
	pop af
	ld (sprite_data),a
	pop af
	push bc
	ld de,sprites
	add a,a
	add a,a
	ld e,a
	ld hl,sprite_data
	ld bc,4
	ldir
	ret

	; Fast 16-bit multiplication.
_mul16:
	ld b,h
	ld c,l
	ld a,16
	ld hl,0
.1:
	srl d
	rr e
	jr nc,.2
	add hl,bc
.2:	sla c
	rl b
	dec a
	jp nz,.1
	ret

	; 16-bit signed modulo.
	; hl = hl % de
_mod16s:
	ld a,h
	or a
	push af
	bit 7,h
	call nz,_neg16
	ex de,hl
	bit 7,h
	call nz,_neg16
	ex de,hl
	call _mod16
	pop af
	ret p
	jp _neg16

	; 16-bit signed division.
	; hl = hl / de
_div16s:
	ld a,h
	xor d
	push af
	bit 7,h
	call nz,_neg16
	ex de,hl
	bit 7,h
	call nz,_neg16
	ex de,hl
	call _div16
	pop af
	ret p
	jp _neg16

_abs16:
	bit 7,h
	ret z
_neg16:
	ld a,h
	cpl
	ld h,a
	ld a,l
	cpl
	ld l,a
	inc hl
	ret

	; Fast 16-bit division.
	; hl = hl / de
_div16:
	ld b,h
	ld c,l
	ld hl,0
	ld a,16
.1:
	rl c
	rl b
	adc hl,hl
	sbc hl,de
	jp nc,.2	
	add hl,de
.2:
	ccf
	dec a
	jp nz,.1
	rl c
	rl b
	ld h,b
	ld l,c
	ret

	; Fast 16-bit modulo.
_mod16:
	ld b,h
	ld c,l
	ld hl,0
	ld a,16
.1:
	rl c
	rl b
	adc hl,hl
	sbc hl,de
	jp nc,.2	
	add hl,de
.2:
	ccf
	dec a
	jp nz,.1
	ret

_sgn16:
	ld a,h
	or l
	ret z
	bit 7,h
	ld hl,$ffff
	ret nz
	inc hl
	inc hl
	ret

	; Random number generator.
	; From my game Mecha Eight.
random:
        ld hl,(lfsr)
        ld a,h
        or l
        jr nz,.0
        ld hl,$7811
.0:     ld a,h
        and $80
        ld b,a
        ld a,h
        and $02
        rrca
        rrca
        xor b
        ld b,a
        ld a,h
        and $01
        rrca
        xor b
        ld b,a
        ld a,l
        and $20
        rlca
        rlca
        xor b
        rlca
        rr h
        rr l
        ld (lfsr),hl
        ret

sn76489_freq:
    if COLECO+SG1000
	ld b,a
	ld a,l
	and $0f
	or b
	out (PSG),a
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,hl
	ld a,h
	and $3f
	out (PSG),a
    endif
	ret

sn76489_vol:
    if COLECO+SG1000
	cpl
	and $0f
	or b
	out (PSG),a
    endif
	ret

sn76489_control:
    if COLECO+SG1000
	and $0f
	or $e0
	out (PSG),a
    endif
	ret

ay3_reg:
    if COLECO
	push af
	ld a,b
	out ($50),a
	pop af
	out ($51),a
	ret
    endif
    if SG1000
        ret
    endif
    if MSX
	ld e,a
	ld a,b
	jp WRTPSG
    endif
    if SVI
	push af
	ld a,b
	out ($88),a
	pop af
	out ($8c),a
	ret
    endif

ay3_freq:
    if COLECO
	out ($50),a
	push af
	ld a,l
	out ($51),a
	pop af
	inc a
	out ($50),a
	push af
	ld a,h
	and $0f
	out ($51),a
	pop af
	ret
    endif
    if SG1000
	ret
    endif
    if MSX
	ld e,l
	call WRTPSG
	ld e,h
	inc a
	jp WRTPSG
    endif
    if SVI
	out ($88),a
	push af
	ld a,l
	out ($8c),a
	pop af
	inc a
	out ($88),a
	push af
	ld a,h
	and $0f
	out ($8c),a
	pop af
	ret
    endif

    if SG1000+SVI
	; Required for SG1000 as it doesn't have a BIOS
	; Required for SVI because we don't have access to BIOS in cartridge.
	;
        ; My personal font for TMS9928.
        ;
        ; Patterned after the TMS9928 programming manual 6x8 letters
        ; with better lowercase letters, also I made a proper
        ; AT sign.
        ;
font_bitmaps:
        db $00,$00,$00,$00,$00,$00,$00,$00      ; $20 space
        db $20,$20,$20,$20,$20,$00,$20,$00      ; $21 !
        db $50,$50,$50,$00,$00,$00,$00,$00      ; $22 "
        db $50,$50,$f8,$50,$f8,$50,$50,$00      ; $23 #
        db $20,$78,$a0,$70,$28,$f0,$20,$00      ; $24 $
        db $c0,$c8,$10,$20,$40,$98,$18,$00      ; $25 %
        db $40,$a0,$40,$a0,$a8,$90,$68,$00      ; $26 &
        db $60,$20,$40,$00,$00,$00,$00,$00      ; $27 '
        db $10,$20,$40,$40,$40,$20,$10,$00      ; $28 (
        db $40,$20,$10,$10,$10,$20,$40,$00      ; $29 )
        db $00,$a8,$70,$20,$70,$a8,$00,$00      ; $2a *
        db $00,$20,$20,$f8,$20,$20,$00,$00      ; $2b +
        db $00,$00,$00,$00,$00,$60,$20,$40      ; $2c ,
        db $00,$00,$00,$fc,$00,$00,$00,$00      ; $2d -
        db $00,$00,$00,$00,$00,$00,$60,$00      ; $2e .
        db $00,$08,$10,$20,$40,$80,$00,$00      ; $2f /
        db $70,$88,$98,$a8,$c8,$88,$70,$00      ; $30 0
        db $20,$60,$20,$20,$20,$20,$f8,$00      ; $31 1
        db $70,$88,$08,$10,$60,$80,$f8,$00      ; $32 2
        db $70,$88,$08,$30,$08,$88,$70,$00      ; $33 3
        db $30,$50,$90,$90,$f8,$10,$10,$00      ; $34 4
        db $f8,$80,$f0,$08,$08,$08,$f0,$00      ; $35 5
        db $30,$40,$80,$f0,$88,$88,$70,$00      ; $36 6
        db $f8,$08,$10,$20,$20,$20,$20,$00      ; $37 7
        db $70,$88,$88,$70,$88,$88,$70,$00      ; $38 8
        db $70,$88,$88,$78,$08,$10,$60,$00      ; $39 9
        db $00,$00,$00,$60,$00,$60,$00,$00      ; $3a :
        db $00,$00,$00,$60,$00,$60,$20,$40      ; $3b ;
        db $10,$20,$40,$80,$40,$20,$10,$00      ; $3c <
        db $00,$00,$f8,$00,$f8,$00,$00,$00      ; $3d =
        db $08,$04,$02,$01,$02,$04,$08,$00      ; $3e >
        db $70,$88,$08,$10,$20,$00,$20,$00      ; $3f ?
        db $70,$88,$98,$a8,$98,$80,$70,$00      ; $40 @
        db $20,$50,$88,$88,$f8,$88,$88,$00      ; $41 A
        db $f0,$88,$88,$f0,$88,$88,$f0,$00      ; $42 B
        db $70,$88,$80,$80,$80,$88,$70,$00      ; $43 C
        db $f0,$88,$88,$88,$88,$88,$f0,$00      ; $44 D
        db $f8,$80,$80,$f0,$80,$80,$f8,$00      ; $45 E
        db $f8,$80,$80,$f0,$80,$80,$80,$00      ; $46 F
        db $70,$88,$80,$b8,$88,$88,$70,$00      ; $47 G
        db $88,$88,$88,$f8,$88,$88,$88,$00      ; $48 H
        db $70,$20,$20,$20,$20,$20,$70,$00      ; $49 I
        db $08,$08,$08,$08,$88,$88,$70,$00      ; $4A J
        db $88,$90,$a0,$c0,$a0,$90,$88,$00      ; $4B K
        db $80,$80,$80,$80,$80,$80,$f8,$00      ; $4C L
        db $88,$d8,$a8,$a8,$88,$88,$88,$00      ; $4D M
        db $88,$c8,$c8,$a8,$98,$98,$88,$00      ; $4E N
        db $70,$88,$88,$88,$88,$88,$70,$00      ; $4F O
        db $f0,$88,$88,$f0,$80,$80,$80,$00      ; $50 P
        db $70,$88,$88,$88,$88,$a8,$90,$68      ; $51 Q
        db $f0,$88,$88,$f0,$a0,$90,$88,$00      ; $52 R
        db $70,$88,$80,$70,$08,$88,$70,$00      ; $53 S
        db $f8,$20,$20,$20,$20,$20,$20,$00      ; $54 T
        db $88,$88,$88,$88,$88,$88,$70,$00      ; $55 U
        db $88,$88,$88,$88,$50,$50,$20,$00      ; $56 V
        db $88,$88,$88,$a8,$a8,$d8,$88,$00      ; $57 W
        db $88,$88,$50,$20,$50,$88,$88,$00      ; $58 X
        db $88,$88,$88,$70,$20,$20,$20,$00      ; $59 Y
        db $f8,$08,$10,$20,$40,$80,$f8,$00      ; $5A Z
        db $78,$60,$60,$60,$60,$60,$78,$00      ; $5B [
        db $00,$80,$40,$20,$10,$08,$00,$00      ; $5C \
        db $F0,$30,$30,$30,$30,$30,$F0,$00      ; $5D ]
        db $20,$50,$88,$00,$00,$00,$00,$00      ; $5E 
        db $00,$00,$00,$00,$00,$00,$f8,$00      ; $5F _
        db $40,$20,$10,$00,$00,$00,$00,$00      ; $60 
        db $00,$00,$68,$98,$88,$98,$68,$00      ; $61 a
        db $80,$80,$f0,$88,$88,$88,$f0,$00      ; $62 b
        db $00,$00,$78,$80,$80,$80,$78,$00      ; $63 c
        db $08,$08,$68,$98,$88,$98,$68,$00      ; $64 d
        db $00,$00,$70,$88,$f8,$80,$70,$00      ; $65 e
        db $30,$48,$40,$e0,$40,$40,$40,$00      ; $66 f
        db $00,$00,$78,$88,$88,$78,$08,$70      ; $67 g
        db $80,$80,$f0,$88,$88,$88,$88,$00      ; $68 h
        db $20,$00,$60,$20,$20,$20,$70,$00      ; $69 i
        db $08,$00,$18,$08,$88,$88,$70,$00      ; $6a j
        db $80,$80,$88,$90,$e0,$90,$88,$00      ; $6b k
        db $60,$20,$20,$20,$20,$20,$70,$00      ; $6c l
        db $00,$00,$d0,$a8,$a8,$a8,$a8,$00      ; $6d m
        db $00,$00,$b0,$c8,$88,$88,$88,$00      ; $6e n
        db $00,$00,$70,$88,$88,$88,$70,$00      ; $6f o
        db $00,$00,$f0,$88,$88,$88,$f0,$80      ; $70 p
        db $00,$00,$78,$88,$88,$88,$78,$08      ; $71 q
        db $00,$00,$b8,$c0,$80,$80,$80,$00      ; $72 r
        db $00,$00,$78,$80,$70,$08,$f0,$00      ; $73 s
        db $20,$20,$f8,$20,$20,$20,$20,$00      ; $74 t
        db $00,$00,$88,$88,$88,$98,$68,$00      ; $75 u
        db $00,$00,$88,$88,$88,$50,$20,$00      ; $76 v
        db $00,$00,$88,$a8,$a8,$a8,$50,$00      ; $77 w
        db $00,$00,$88,$50,$20,$50,$88,$00      ; $78 x
        db $00,$00,$88,$88,$98,$68,$08,$70      ; $79 y
        db $00,$00,$f8,$10,$20,$40,$f8,$00      ; $7a z
        db $18,$20,$20,$40,$20,$20,$18,$00      ; $7b {
        db $20,$20,$20,$20,$20,$20,$20,$00      ; $7c |
        db $c0,$20,$20,$10,$20,$20,$c0,$00      ; $7d } 
        db $00,$00,$40,$a8,$10,$00,$00,$00      ; $7e
        db $70,$70,$20,$f8,$20,$70,$50,$00      ; $7f
    endif

vdp_generic_mode:
	call nmi_off
	call WRTVDP
	ld bc,$a201
	call WRTVDP
	ld bc,$0602	; $1800 for pattern table.
	call WRTVDP
	ld b,d
	ld c,$03	; for color table.
	call WRTVDP
	ld b,e
	ld c,$04	; for bitmap table.
	call WRTVDP
	ld bc,$3605	; $1b00 for sprite attribute table.
	call WRTVDP
	ld bc,$0706	; $3800 for sprites bitmaps.
	call WRTVDP
	ld bc,$0107
	jp WRTVDP

mode_0:
	ld hl,mode
	res 2,(hl)
	ld bc,$0200
	ld de,$ff03	; $2000 for color table, $0000 for bitmaps.
	call vdp_generic_mode
    if COLECO
	ld hl,($006c)
	ld de,-128
	add hl,de
    endif
    if SG1000+SVI
	ld hl,font_bitmaps
    endif
    if MSX
	ld hl,($0004)   
	inc h
    endif
	ld de,$0100
	ld bc,$0300
	call LDIRVM3
	call nmi_on
	call nmi_off
	ld hl,$2000
	ld bc,$1800
	ld a,$f0
	call FILVRM
	call nmi_on
	call cls
vdp_generic_sprites:
	call nmi_off
	ld hl,$1b00
	ld bc,$0080
	ld a,$d1
	call FILVRM
	ld hl,sprites
	ld de,sprites+1
	ld bc,127
	ld (hl),$d1
	ldir
	call nmi_on
	call nmi_off
	ld bc,$e201	; Enable screen and interrupts.
	call WRTVDP
	jp nmi_on

mode_1:
	ld hl,mode
	res 2,(hl)
	ld bc,$0200
	ld de,$ff03	; $2000 for color table, $0000 for bitmaps.
	call vdp_generic_mode
	ld hl,$0000
	ld bc,$1800
	xor a
	call FILVRM
	call nmi_on
	call nmi_off
	ld hl,$2000
	ld bc,$1800
	ld a,$f0
	call FILVRM
	call nmi_on
	ld hl,$1800
.1:	call nmi_off
	ld b,32
.2:	ld a,l
	call WRTVRM
	inc hl
	djnz .2
	call nmi_on
	ld a,h
	cp $1b
	jp nz,.1
	jp vdp_generic_sprites

mode_2:
	ld hl,mode
	set 2,(hl)
	ld bc,$0000
	ld de,$8000	; $2000 for color table, $0000 for bitmaps.
	call vdp_generic_mode
    if COLECO
	ld hl,($006c)
	ld de,-128
	add hl,de
    endif
    if SG1000+SVI
	ld hl,font_bitmaps
    endif
    if MSX
	ld hl,($0004)   
	inc h
    endif
	ld de,$0100
	ld bc,$0300
	call LDIRVM
	call nmi_on
	call nmi_off
	ld hl,$2000
	ld bc,$0020
	ld a,$f0
	call FILVRM
	call nmi_on
	call cls
	jp vdp_generic_sprites

    if MSX
ENASLT: EQU $0024       ; Select slot (H=Addr, A=Slot)
RSLREG: EQU $0138       ; Read slot status in A

        ;
        ; Get slot mapping
        ; B = 16K bank (0 for $0000, 1 for $4000, 2 for $8000, 3 for $c000)
        ; A = Current slot selection status (CALL RSLREG)
        ;
get_slot_mapping:
        call rotate_slot
        ld c,a
        add a,$C1       ; EXPTBL
        ld l,a
        ld h,$FC
        ld a,(hl)
        and $80         ; Get expanded flag
        or c
        ld c,a
        inc hl
        inc hl
        inc hl
        inc hl
        ld a,(hl)       ; SLTTBL
        call rotate_slot
        rlca
        rlca
        or c            ; A contains bit 7 = Marks expanded
                        ;            bit 6 - 4 = Doesn't care
                        ;            bit 3 - 2 = Secondary mapper
                        ;            bit 1 - 0 = Primary mapper
        ret

rotate_slot:
        push bc
        dec b
        inc b
        jr z,.1
.0:     rrca
        rrca
        djnz .0
.1:     and 3
        pop bc
        ret

    endif

nmi_handler:
	push af
	push hl
	ld hl,mode
	bit 0,(hl)
	jr z,.1
	set 1,(hl)
	pop hl
	pop af
	retn

.0:	res 1,(hl)

.1:	push bc
	push de
  if CVBASIC_BANK_SWITCHING
    if COLECO
	ld a,($ffbf)
    endif
    if SG1000
	ld a,($7fff)
    endif
    if MSX
	ld a,($bfff)
    endif
	push af
  endif
    if SG1000+MSX+SVI
	in a,(VDPR+1)
    endif
	ld bc,$8000+VDP
	bit 2,(hl)
	jr z,.4

	ld hl,$1b00
	call SETWRT
	ld hl,sprites
	outi
	jp nz,$-2
	jr .5

.4:
	ld hl,$1b00
	call SETWRT
	ld a,(flicker)
	add a,$04
	ld (flicker),a
	ld l,a
	ld h,sprites>>8
	ld de,24
	ld b,128
.6:
	res 7,l
	outi
	jp $+3
	outi
	jp $+3
	outi
	jp $+3
	outi
	jp $+3
	add hl,de
	jp nz,.6
.5:

    if COLECO
	out (JOYSEL),a
	ex (sp),hl
	ex (sp),hl
	in a,(JOY1)
	or $b0
	ld b,a
	in a,(JOY2)
	or $b0
	ld c,a

	out (KEYSEL),a
	ex (sp),hl
	ex (sp),hl
	in a,(JOY1)
	ld d,a
	in a,(JOY2)
	ld e,a

	ld a,d
	rlca
	or $7f
	and b
	cpl
	ld (joy1_data),a

	ld a,e
	rlca
	or $7f
	and c
	cpl
	ld (joy2_data),a

	ld a,d
	and $0f
	ld c,a
	ld b,0
	ld hl,keypad_table
	add hl,bc
	ld a,(hl)
	ld (key1_data),a

	ld a,e
	and $0f
	ld c,a
	ld hl,keypad_table
	add hl,bc
	ld a,(hl)
	ld (key2_data),a
    endif
    if SG1000
	ld a,$07
	out ($de),a
	ld b,$ff
	in a,(JOY1)
	ld h,a
	in a,(JOY2)
	ld l,a
	in a,($de)
	cp 7
	jr nz,.sg1000
	ld a,$04
	out ($de),a
	in a,($dc)
	bit 5,a		; Keyboard down.
	jr nz,$+4
	res 1,h
	ld a,$05
	out ($de),a
	in a,($dc)
	bit 5,a		; Keyboard left.
	jr nz,$+4
	res 2,h
	ld a,$06
	out ($de),a
	in a,($dc)
	bit 5,a		; Keyboard right.
	jr nz,$+4
	res 3,h
	bit 6,a		; Keyboard up.
	jr nz,$+4
	res 0,h
	ld a,$02
	out ($de),a
	in a,($dc)
	bit 4,a		; Keyboard Ins
	jr nz,$+4
	res 4,h
	ld a,$03
	out ($de),a
	in a,($dc)
	bit 4,a		; Keyboard Del
	jr nz,$+4
	res 5,h
.sg1000:
        bit 0,h
        jr nz,$+4
        res 0,b
        bit 1,h
        jr nz,$+4
        res 2,b
        bit 2,h
        jr nz,$+4
        res 3,b
        bit 3,h
        jr nz,$+4
        res 1,b
        bit 4,h
        jr nz,$+4
        res 6,b
        bit 5,h
        jr nz,$+4
        res 7,b
	ld a,b
	cpl
	ld (joy1_data),a

	ld a,$ff
        bit 6,h
        jr nz,$+4
        res 0,a
        bit 7,h
        jr nz,$+4
        res 2,a

        bit 0,l
        jr nz,$+4
        res 3,a
        bit 1,l
        jr nz,$+4
        res 1,a
        bit 2,l
        jr nz,$+4
        res 4,a
        bit 3,l
        jr nz,$+4
        res 5,a
	cpl
	ld (joy2_data),a

    endif
    if MSX

	ld a,15
	call RDPSG
	and $b0
	or $4f
	ld e,a
	ld a,15
	call WRTPSG
	ld a,14
	call RDPSG
	ld b,$ff
	bit 0,a
	jr nz,$+4
	res 0,b
	bit 3,a
	jr nz,$+4
	res 1,b
	bit 1,a
	jr nz,$+4
	res 2,b
	bit 2,a
	jr nz,$+4
	res 3,b
	bit 4,a
	jr nz,$+4
	res 6,b
	bit 5,a
	jr nz,$+4
	res 7,b
	ld a,b
	cpl
	ld (joy2_data),a

	in a,($aa)
	and $f0
	or $00
	out ($aa),a
	in a,($a9)
	cp $ff
	ld c,$00
	ld b,$08
	jr nz,.key1
	in a,($aa)
	and $f0
	or $01
	out ($aa),a
	in a,($a9)
	and $0f
	cp $0f
	jr z,.key2
	ld c,$08
	ld b,$04
.key1:	rra
	inc c
	jr c,.key1
	ld a,c
	dec a
.key2:
	ld (key1_data),a	

        ld b,$ff
	in a,($aa)
	and $f0
	or $08
	out ($aa),a
	in a,($a9)
	bit 5,a
	jr nz,$+4
        res 0,b
	bit 7,a
	jr nz,$+4
        res 1,b
        bit 6,a
        jr nz,$+4
        res 2,b
        bit 4,a
        jr nz,$+4
        res 3,b
	bit 0,a
	jr nz,$+4
	res 6,b
	in a,($aa)
	and $f0
	or $04
	out ($aa),a
	in a,($a9)
	bit 2,a
	jr nz,$+4
	res 7,b

	ld a,15
	call RDPSG
	and $b0
	or $0f
	ld e,a
	ld a,15
	call WRTPSG
	ld a,14
	call RDPSG
	bit 0,a
	jr nz,$+4
	res 0,b
	bit 3,a
	jr nz,$+4
	res 1,b
	bit 1,a
	jr nz,$+4
	res 2,b
	bit 2,a
	jr nz,$+4
	res 3,b
	bit 4,a
	jr nz,$+4
	res 6,b
	bit 5,a
	jr nz,$+4
	res 7,b

	ld a,b
	cpl
	ld (joy1_data),a
    endif
    if SVI
	ld a,14
	call RDPSG
	ld b,$ff
	bit 4,a
	jr nz,$+4
	res 0,b
	bit 7,a
	jr nz,$+4
	res 1,b
	bit 5,a
	jr nz,$+4
	res 2,b
	bit 6,a
	jr nz,$+4
	res 3,b

	in a,($98)
	bit 5,a
	jr nz,$+4
	res 6,b
	ld a,b
	cpl
	ld (joy2_data),a

	ld a,$10
	out ($96),a
	in a,($99)
	cp $ff
	ld c,$00
	ld b,$08
	jr nz,.key1
	ld a,$11
	out ($96),a
	in a,($99)
	and $0f
	cp $0f
	jr z,.key2
	ld c,$08
	ld b,$04
.key1:	rra
	inc c
	jr c,.key1
	ld a,c
	dec a
.key2:
	ld (key1_data),a	

        ld b,$ff
	ld a,$15
	out ($96),a
	in a,($99)
	bit 7,a
	jr nz,$+4
        res 0,b
	ld a,$18
	out ($96),a
	in a,($99)
	bit 7,a
	jr nz,$+4
        res 1,b
	ld a,$17
	out ($96),a
	in a,($99)
	bit 7,a
        jr nz,$+4
        res 2,b
	ld a,$16
	out ($96),a
	in a,($99)
	bit 7,a
        jr nz,$+4
        res 3,b
	ld a,$18
	out ($96),a
	in a,($99)
	bit 0,a
	jr nz,$+4
	res 6,b
	ld a,$13
	out ($96),a
	in a,($99)
	bit 5,a
	jr nz,$+4
	res 7,b

	ld a,14
	call RDPSG
	bit 0,a
	jr nz,$+4
	res 0,b
	bit 3,a
	jr nz,$+4
	res 1,b
	bit 1,a
	jr nz,$+4
	res 2,b
	bit 2,a
	jr nz,$+4
	res 3,b
	
	in a,($98)
	bit 4,a
	jr nz,$+4
	res 6,b
	ld a,b
	cpl
	ld (joy1_data),a
    endif

    if CVBASIC_MUSIC_PLAYER
	ld a,(music_mode)
	or a
	call nz,music_hardware
    endif

	ld hl,(frame)
	inc hl
	ld (frame),hl

	ld hl,lfsr	; Make LFSR more random
	inc (hl)
	inc (hl)
	inc (hl)

    if CVBASIC_MUSIC_PLAYER
	;
	; Music is played with a 50hz clock.
	;
	ld a,(ntsc)
	or a
	jr z,.2
	ld a,(music_tick)
	inc a
	cp 6
	jr nz,$+3
	xor a
	ld (music_tick),a
	jr z,.3
.2:
	ld a,(music_mode)
	or a
	call nz,music_generate
.3:
    endif

  if CVBASIC_BANK_SWITCHING
	pop af
    if COLECO
	ld l,a
	ld h,$ff
	ld a,(hl)
    endif
    if SG1000
	ld ($fffe),a
    endif
    if MSX
	ld ($7000),a
    endif
  endif
	pop de
	pop bc
	pop hl
    if COLECO
	in a,(VDP+1)
	pop af
	retn
    endif
    if SG1000+SVI
	pop af
        ei
        reti
    endif
    if MSX
	pop af
        ret
    endif

	;
	; The music player code comes from my
	; game Princess Quest for Colecovision (2012)
	;

        ;
        ; Init music player.
        ;
music_init:
    if COLECO+SG1000
        ld a,$9f
        out (PSG),a
        ld a,$bf
        out (PSG),a
        ld a,$df
        out (PSG),a
        ld a,$ff
        out (PSG),a
        ld a,$ec
        out (PSG),a
    endif
    if MSX+SVI
	ld a,$08
	ld e,$00
	call WRTPSG
	ld a,$09
	ld e,$00
	call WRTPSG
	ld a,$0a
	ld e,$00
	call WRTPSG
	ld a,$07
	ld e,$b8
	call WRTPSG
    endif
    if SGM
	ld b,$08
	xor a
	call ay3_reg
	ld b,$09
	call ay3_reg
	ld b,$0a
	call ay3_reg
	ld b,$07
	ld a,$b8
	call ay3_reg
    endif
    if CVBASIC_MUSIC_PLAYER
    else
	ret
    endif

    if CVBASIC_MUSIC_PLAYER
        ld a,$ff
        ld (audio_vol4hw),a
        ld a,$ec
        ld (audio_control),a
        ld a,$b8
        ld (audio_mix),a
	ld hl,music_silence
        ;
	; Play a music.
	; HL = Pointer to music.
        ;
music_play:
        call nmi_off
        ld a,(hl)          
        ld (music_timing),a
        inc hl
        ld (music_start),hl
        ld (music_pointer),hl
        xor a
        ld (music_note_counter),a
	inc a
	ld (music_playing),a
  if CVBASIC_BANK_SWITCHING
    if COLECO
	ld a,($ffbf)
    endif
    if SG1000
        ld a,($7fff)
    endif
    if MSX
        ld a,($bfff)
    endif
        ld (music_bank),a
  endif
        jp nmi_on

        ;
        ; Generates music.
        ;
music_generate:
        ld a,(audio_mix)
        and $c0                 
        or $38
        ld (audio_mix),a
        xor a                ; Turn off all the sound channels.
        ld l,a
        ld h,a
        ld (audio_vol1),hl   ; audio_vol1/audio_vol2
        ld (audio_vol3),a
	ld a,$ff
	ld (audio_vol4hw),a

        ld a,(music_note_counter)
        or a
        jp nz,.6
        ld hl,(music_pointer)
.15:    push hl
  if CVBASIC_BANK_SWITCHING
	ld a,(music_bank)
    if COLECO
	ld e,a
	ld d,$ff
	ld a,(de)
    endif
    if SG1000
        ld ($fffe),a
    endif
    if MSX
	ld ($7000),a
    endif
  endif
        ld b,(hl)
        inc hl
        ld c,(hl)
        inc hl
        ld d,(hl)
        inc hl
        ld e,(hl)
        pop hl
        ld a,(music_timing)
        rlca
        jr nc,.16
        ld e,d
        ld d,0
        jr .17

.16:    rlca
        jr nc,.17
        ld e,0
.17:    ld a,b		; Read first byte.
        cp -2           ; End of music?
        jr nz,.19       ; No, jump.
        xor a		; Keep at same place.
        ld (music_playing),a
        ret

.19:    cp -3           ; Repeat music?
        jp nz,.0
        ld hl,(music_start)
        jr .15

.0:     ld a,(music_timing)
        and $3f         ; Restart note time.
        ld (music_note_counter),a
        ld a,b
        cp $3f          ; Sustain?
        jr z,.1
        rlca
        rlca
        and 3
        ld (music_instrument_1),a    
        ld a,b
        and $3f
        ld (music_note_1),a    
        xor a         
        ld (music_counter_1),a    
.1:     ld a,c          
        cp $3f          
        jr z,.2
        rlca
        rlca
        and 3
        ld (music_instrument_2),a    
        ld a,c
        and $3f
        ld (music_note_2),a    
        xor a         
        ld (music_counter_2),a    
.2:     ld a,d          
        cp $3f          
        jr z,.3
        rlca
        rlca
        and 3
        ld (music_instrument_3),a    
        ld a,d
        and $3f
        ld (music_note_3),a    
        xor a         
        ld (music_counter_3),a    
.3:     ld a,e          
        ld (music_drum),a
        xor a
        ld (music_counter_4),a
        inc hl
        inc hl
        inc hl
        ld a,(music_timing)
        and $c0
        jr nz,.14
        inc hl
.14:    ld (music_pointer),hl

.6:     ld a,(music_note_1)    
        or a            
        jr z,.7         
        ld bc,(music_instrument_1)
        call music_note2freq
        ld (audio_freq1),hl 
        ld (audio_vol1),a

.7:     ld a,(music_note_2)    
        or a            
        jr z,.8         
        ld bc,(music_instrument_2)
        call music_note2freq
        ld (audio_freq2),hl 
        ld (audio_vol2),a

.8:     ld a,(music_note_3)    
        or a            
        jr z,.9         
        ld bc,(music_instrument_3)
        call music_note2freq
        ld (audio_freq3),hl 
        ld (audio_vol3),a

.9:     ld a,(music_drum)    
        or a            
        jr z,.4         
        dec a           ; 1 - Long drum.
        jr nz,.5
        ld a,(music_counter_4)
        cp 3
        jp nc,.4
.10:    ld a,5
        ld (audio_noise),a
        call enable_drum
        jr .4

.5:     dec a           ; 2 - Short durm.
        jr nz,.11
        ld a,(music_counter_4)
        or a
        jp nz,.4
        ld a,8
        ld (audio_noise),a
        call enable_drum
        jr .4

.11:    ;dec a           ; 3 - Roll.
        ;jp nz,.4
        ld a,(music_timing)
        and $3e
        rrca
        ld b,a
        ld a,(music_counter_4)
        cp 2
        jp c,.10
        cp b
        jp c,.4
        dec a
        dec a
        cp b
        jp c,.10
.4:
        ld a,(music_counter_1)
        inc a
        cp $18
        jp nz,$+5
        sub $08
        ld (music_counter_1),a

        ld a,(music_counter_2)
        inc a
        cp $18
        jp nz,$+5
        sub $08
        ld (music_counter_2),a

        ld a,(music_counter_3)
        inc a
        cp $18
        jp nz,$+5
        sub $08
        ld (music_counter_3),a

        ld hl,music_counter_4
        inc (hl)
        ld hl,music_note_counter
        dec (hl)
        ret

        ;
        ; Converts note to frequency.
 	; Input:
	;   A = Note (1-62).
	;   B = Instrument counter.
	;   C = Instrument.
        ; Output:
	;   HL = Frequency.
	;   A = Volume.
	;
music_note2freq:
        add a,a
        ld e,a
        ld d,0
        ld hl,music_notes_table
        add hl,de
        ld a,(hl)
        inc hl
        ld h,(hl)
        ld l,a
        ld a,c
        or a
        jp z,music_piano
        dec a
        jp z,music_clarinet
        dec a
        jp z,music_flute
        ;
        ; Bass instrument.
        ;
music_bass:
        add hl,hl

        ;
        ; Piano instrument.
        ;
music_piano:
        ld a,b
        add a,.1&255
        ld c,a
        adc a,.1>>8
        sub c
        ld b,a
        ld a,(bc)
        ret

.1:
        db 12,11,11,10,10,9,9,8
        db 8,7,7,6,6,5,5,4
        db 4,4,5,5,4,4,3,3

        ;
        ; Clarinet instrument.
        ;
music_clarinet:
        ld a,b
        add a,.1&255
        ld c,a
        adc a,.1>>8
        sub c
        ld b,a
        ld a,(bc)
        ld e,a
        rlca
        sbc a,a
        ld d,a
        add hl,de
        srl h           
        rr l
        jp nc,.2
        inc hl
.2:     ld a,c
        add a,24
        ld c,a
	jr nc,$+3
	inc b
        ld a,(bc)
        ret

.1:
        db 0,0,0,0
        db -2,-4,-2,0
        db 2,4,2,0
        db -2,-4,-2,0
        db 2,4,2,0
        db -2,-4,-2,0

        db 13,14,14,13,13,12,12,12
        db 11,11,11,11,12,12,12,12
        db 11,11,11,11,12,12,12,12

        ;
        ; Flute instrument.
        ;
music_flute:
        ld a,b
        add a,.1&255
        ld c,a
        adc a,.1>>8
        sub c
        ld b,a
        ld a,(bc)
        ld e,a
        rlca
        sbc a,a
        ld d,a
        add hl,de
        ld a,c
        add a,24
        ld c,a
	jr nc,$+3
	inc b
        ld a,(bc)
        ret

.1:
        db 0,0,0,0
        db 0,1,2,1
        db 0,1,2,1
        db 0,1,2,1
        db 0,1,2,1
        db 0,1,2,1
                 
        db 10,12,13,13,12,12,12,12
        db 11,11,11,11,10,10,10,10
        db 11,11,11,11,10,10,10,10

        ;
        ; Emit sound.
        ;
music_hardware:
    if COLECO+SG1000
	ld a,(music_mode)
	cp 4		; PLAY SIMPLE?
	jr c,.7		; Yes, jump.
        ld a,(audio_vol2)
        or a
        jp nz,.7
        ld a,(audio_vol3)
        or a
        jp z,.7
        ld (audio_vol2),a
        xor a
        ld (audio_vol3),a
        ld hl,(audio_freq3)
        ld (audio_freq2),hl
.7:
        ld hl,(audio_freq1)
        ld a,h
        cp 4
        ld a,$9f
        jp nc,.1
        ld a,l
        and $0f
        or $80
        out (PSG),a
        add hl,hl
        add hl,hl
        add hl,hl
        add hl,hl
        ld a,h
        out (PSG),a
        ld a,(audio_vol1)
        add a,ay2sn&255
        ld l,a
        adc a,ay2sn>>8
        sub l
        ld h,a
        ld a,(hl)
        or $90
.1:     out (PSG),a

        ld hl,(audio_freq2)
        ld a,h
        cp 4
        ld a,$bf
        jp nc,.2
        ld a,l
        and $0f
        or $a0
        out (PSG),a
        add hl,hl
        add hl,hl
        add hl,hl
        add hl,hl
        ld a,h
        out (PSG),a
        ld a,(audio_vol2)
        add a,ay2sn&255
        ld l,a
        adc a,ay2sn>>8
        sub l
        ld h,a
        ld a,(hl)
        or $b0
.2:     out (PSG),a

	ld a,(music_mode)
	cp 4		; PLAY SIMPLE?
	jr c,.6		; Yes, jump.

        ld hl,(audio_freq3)
        ld a,h
        cp 4
        ld a,$df
        jp nc,.3
        ld a,l
        and $0f
        or $c0
        out (PSG),a
        add hl,hl
        add hl,hl
        add hl,hl
        add hl,hl
        ld a,h
        out (PSG),a
        ld a,(audio_vol3)
        add a,ay2sn&255
        ld l,a
        adc a,ay2sn>>8
        sub l
        ld h,a
        ld a,(hl)
        or $d0
.3:     out (PSG),a

.6:
	ld a,(music_mode)
	and 1		; NO DRUMS?
	ret z		; Yes, return.

        ld a,(audio_vol4hw)
        inc a           
        jr z,.4        
        ld a,(audio_noise)
        cp 16
        ld b,$ec        
        jp c,.5
        ld b,$ed        
;       ld b,$ee        
.5:     ld a,(audio_control)
        cp b
        jr z,.4
        ld a,b
        ld (audio_control),a
        out (PSG),a
.4:     ld a,(audio_vol4hw)
        out (PSG),a
        ret
    endif
    if MSX+SVI
	ld a,(music_mode)
	cp 4		; PLAY SIMPLE?
	jr c,.8		; Yes, jump.	
	ld hl,audio_freq1
	ld bc,$0b00
	ld a,c
	ld e,(hl)
	call WRTPSG
	inc hl
	inc c
	djnz $-7
	ret
.8:
	ld hl,audio_freq1
	ld bc,$0400
	ld a,c
	ld e,(hl)
	call WRTPSG
	inc hl
	inc c
	djnz $-7
	inc hl
	inc hl
	inc c
	inc c
	ld a,(music_mode)
	and 1
	jr z,.9
	ld a,c
	ld e,(hl)
	call WRTPSG
	inc hl
	inc c
	ld a,c
	ld e,(hl)
	call WRTPSG
	inc hl
	inc c
	jr .10
.9:	inc hl
	inc c
	inc hl
	inc c
.10:	ld b,$02
	ld a,c
	ld e,(hl)
	call WRTPSG
	inc hl
	inc c
	djnz $-7
	ret
    endif

        ;
        ; Enable drum.
        ;
enable_drum:
    if COLECO+SG1000
        ld a,$f5
        ld (audio_vol4hw),a
    else
        ld hl,audio_mix
        ld a,(audio_vol2)
        or a
        jr nz,.1
        ld a,10
        ld (audio_vol2),a
        set 1,(hl)
.1:     res 4,(hl)
    endif
        ret

        ;
	; Musical notes table.
	;
music_notes_table:
        ; Silence - 0
        dw 0
        ; 2nd octave - 1
        dw 1721,1621,1532,1434,1364,1286,1216,1141,1076,1017,956,909
        ; 3rd octave - 13
        dw 854,805,761,717,678,639,605,571,538,508,480,453
        ; 4th octave - 25
        dw 427,404,380,360,339,321,302,285,270,254,240,226
        ; 5th octave - 37
        dw 214,202,191,180,170,160,151,143,135,127,120,113
        ; 6th octave - 49
        dw 107,101,95,90,85,80,76,71,67,64,60,57
        ; 7th octave - 61
	dw 54,51,48

    if COLECO+SG1000
        ;
        ; Converts AY-3-8910 volume to SN76489
        ;
ay2sn:
        db $0f,$0f,$0f,$0e,$0e,$0e,$0d,$0b,$0a,$08,$07,$05,$04,$03,$01,$00
    endif

music_silence:
	db 8
	db 0,0,0,0
	db -2
    endif

    if CVBASIC_COMPRESSION
define_char_unpack:
	ex de,hl
	pop af
	pop hl
	push af
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	ex de,hl
	ld a,(mode)
	and 4
	jp z,unpack3
	jp unpack

define_color_unpack:
	ex de,hl
	pop af
	pop hl
	push af
	add hl,hl	; x2
	add hl,hl	; x4
	add hl,hl	; x8
	ex de,hl
	set 5,d
unpack3:
	call .1
	call .1
.1:
	push de
	push hl
	call unpack
	pop hl
	pop de
	ld a,d
	add a,8	
	ld d,a
	ret
	
        ;
        ; Pletter-0.5c decompressor (XL2S Entertainment & Team Bomba)
        ;
unpack:
; Initialization
        ld a,(hl)
        inc hl
	exx
        ld de,0
        add a,a
        inc a
        rl e
        add a,a
        rl e
        add a,a
        rl e
        rl e
        ld hl,.modes
        add hl,de
        ld c,(hl)
        inc hl
        ld b,(hl)
        push bc
        pop ix
        ld e,1
	exx
        ld iy,.loop

; Main depack loop
.literal:
        ex af,af'
        call nmi_off
        ld a,(hl)
        ex de,hl
        call WRTVRM
        ex de,hl
        inc hl
        inc de
        call nmi_on
        ex af,af'
.loop:   add a,a
        call z,.getbit
        jr nc,.literal

; Compressed data
	exx
        ld h,d
        ld l,e
.getlen: add a,a
        call z,.getbitexx
        jr nc,.lenok
.lus:    add a,a
        call z,.getbitexx
        adc hl,hl
        ret c   
        add a,a
        call z,.getbitexx
        jr nc,.lenok
        add a,a
        call z,.getbitexx
        adc hl,hl
        ret c  
        add a,a
        call z,.getbitexx
        jr c,.lus
.lenok:  inc hl
	exx
        ld c,(hl)
        inc hl
        ld b,0
        bit 7,c
        jr z,.offsok
        jp (ix)

.mode6:  add a,a
        call z,.getbit
        rl b
.mode5:  add a,a
        call z,.getbit
        rl b
.mode4:  add a,a
        call z,.getbit
        rl b
.mode3:  add a,a
        call z,.getbit
        rl b
.mode2:  add a,a
        call z,.getbit
        rl b
        add a,a
        call z,.getbit
        jr nc,.offsok
        or a
        inc b
        res 7,c
.offsok: inc bc
        push hl
	exx
        push hl
	exx
        ld l,e
        ld h,d
        sbc hl,bc
        pop bc
        ex af,af'
.loop2: 
        call nmi_off
        call RDVRM              ; unpack
        ex de,hl
        call WRTVRM
        ex de,hl        ; 4
        call nmi_on
        inc hl          ; 6
        inc de          ; 6
        dec bc          ; 6
        ld a,b          ; 4
        or c            ; 4
        jr nz,.loop2     ; 10
        ex af,af'
        pop hl
        jp (iy)

.getbit: ld a,(hl)
        inc hl
	rla
	ret

.getbitexx:
	exx
        ld a,(hl)
        inc hl
	exx
	rla
	ret

.modes:
        dw      .offsok
        dw      .mode2
        dw      .mode3
        dw      .mode4
        dw      .mode5
        dw      .mode6

    endif

START:
    if SVI+SG1000
	im 1
    endif
    if SVI
	ld e,$00
	ld a,$08
	call WRTPSG
	ld a,$09
	call WRTPSG
	ld a,$0A
	call WRTPSG
	ld a,$07
	ld e,$b8
	call WRTPSG
	ld a,$92	; Setup 8255 for keyboard/joystick reading.
	out ($97),a
    endif
    if SG1000
	; Contributed by SiRioKD
	ld a,$9F	; Turn off PSG
	out (PSG),a
	ld a,$BF	
	out (PSG),a
	ld a,$DF
	out (PSG),a
	ld a,$FF
	out (PSG),a	
	ld a,$92	; Setup 8255 for SC3000.
	out ($df),a
    endif
    if SG1000+SVI
	; Wait for VDP ready (around 1000 ms)
	ld b,11
	ld de,$FFFF
.delay1:
	ld hl,$39DE
.delay2:
	add hl,de
	jr c,.delay2
	djnz .delay1
    else
	di
	ld sp,STACK
    endif
	in a,(VDPR+1)
	ld bc,$8201
	call WRTVDP
	in a,(VDPR+1)
	ld bc,$8201
	call WRTVDP
  if CVBASIC_BANK_SWITCHING
    if COLECO
	ld a,($ffc0)	; Megacart
    endif
    if SG1000
        ld a,1		; Sega mapper
        ld ($fffe),a
    endif
    if MSX
        ld a,1		; ASCII 16K
        ld ($7000),a
    endif
  endif
	ld hl,(lfsr)	; Save RAM trash for random generator.
	ld de,BASE_RAM
	xor a
	ld (de),a
	inc de
	bit 2,d
	jp z,$-4
	ld (lfsr),hl

    if SGM
WRITE_REGISTER:	equ $1fd9
FILL_VRAM:	equ $1f82
WRITE_VRAM:	equ $1fdf

        ld b,$00	; First step.
.0:     ld hl,$2000	; RAM at $2000.
.1:     ld (hl),h	; Try to write a byte.
        inc h
        jp p,.1		; Repeat until reaching $8000.
        ld h,$20	; Go back at $2000.
.2:     ld a,(hl)	; Read back byte.
        cp h		; Is it correct?
        jr nz,.3	; No, jump.
        inc h
        jp p,.2		; Repeat until reaching $8000.
        jp .4		; Memory valid!

.3:     ld a,$01        ; Enable SGM
        out ($53),a
        inc b
        bit 1,b         ; Already enabled?
        jr z,.0		; No, test RAM again.

        ld bc,$0000
        call WRITE_REGISTER
        ld bc,$0180
        call WRITE_REGISTER
        ld bc,$0206
        call WRITE_REGISTER
        ld bc,$0380
        call WRITE_REGISTER
        ld bc,$0400
        call WRITE_REGISTER
        ld bc,$0536
        call WRITE_REGISTER
        ld bc,$0607
        call WRITE_REGISTER
        ld bc,$070D
        call WRITE_REGISTER
        ld bc,$03F0
        ld de,$00E8 
        ld hl,$158B     ; Note! direct access to Colecovision ROM
        call WRITE_VRAM
        ld hl,$2000
        ld de,32 
        ld a,$FD
        call FILL_VRAM
        ld hl,$1B00
        ld de,128
        ld a,$D1
        call FILL_VRAM
        ld hl,$1800
        ld de,769
        ld a,$20
        call FILL_VRAM
        ld bc,$0020
        ld de,$1980 
        ld hl,.5
        call WRITE_VRAM
        ld bc,$01C0
        call WRITE_REGISTER
        jr $

.5:     db " SUPER GAME MODULE NOT DETECTED "

.4:
	ld ix,(lfsr)
        ld hl,$2000
        ld de,$2001
        ld bc,$5FFF
        ld (hl),0
        ldir
	ld (lfsr),ix
    endif
    if COLECO
	ld a,($0069)
	cp 50
	ld a,1
	jr nz,$+3
	dec a
    endif
    if SG1000+SVI
	ld a,1
    endif
    if MSX
        call RSLREG
        ld b,1          ; $4000-$7fff
        call get_slot_mapping
        ld h,$80
        call ENASLT     ; Map into $8000-$BFFF

	ld a,($002b)
	cpl
	rlca
	and $01
    endif
	ld (ntsc),a

	call music_init

	xor a
	ld (mode),a

	call mode_0

	ld a,$ff
	ld (joy1_data),a
	ld (joy2_data),a
	ld a,$0f
	ld (key1_data),a
	ld (key2_data),a

    if MSX
	ld hl,nmi_handler
	ld ($fd9b),hl
	ld a,$c3
	ld ($fd9a),a
    endif

	; CVBasic program start.
	; ' Invader for Colecovision - cvbasic
	; ' Started 7/5/2024
	; ' Brian Kumanchik
	; 
	; 
	; ' cvbasic invader.bas invader.asm
	; ' gasm80 invader.asm -o invader.rom
	; ' drag and drop invader.rom on to ColEm to run
	; 
	; 
	; ' colecovision resolution 256 x 196 - 32 col x 24 row
	; ' Z = button 1, A = button 2
	; ' x = 255, y = 190 to hide sprite
	; 
	; 	
	; 	
	; 
	; 	
	; 	define sprite 0,8,sprites 	' sprites include at bottom of code
	LD HL,0
	PUSH HL
	LD A,8
	LD HL,cvb_SPRITES
	CALL define_sprite
	; 	
	; 	
	; 	gosub init_vars				' initialize variables
	CALL cvb_INIT_VARS
	; 	
	; 	
	; 	' array to hold pointer for fake random invader restart x position
	; 	dim r(5) ' uses [array_pointer] variable for index
	; 	r(0) = 83     
	LD A,83
	LD (array_R),A
	; 	r(1) = 212     
	LD A,212
	LD (array_R+1),A
	; 	r(2) = 126    
	LD A,126
	LD (array_R+2),A
	; 	r(3) = 40    
	LD A,40
	LD (array_R+3),A
	; 	r(4) = 169    
	LD A,169
	LD (array_R+4),A
	; 
	; 	
	; ' main loop -----------------------------------------------------------
	; main_loop:
cvb_MAIN_LOOP:
	; 	
	; 	wait ' wait for next frame	
	HALT
	; 	
	; 	
	; 	
	; 	
	; 	' --- GAME OVER state 1 - game over ---
	; 	if game_over = 1 then		
	LD A,(cvb_GAME_OVER)
	CP 1
	JP NZ,cv1
	; 		print at (11*32) + 11,"GAME OVER!"
	LD HL,363
	LD (cursor),HL
	LD HL,cv2
	LD A,10
	CALL print_string
	JP cv3
cv2:
	DB $47,$41,$4d,$45,$20,$4f,$56,$45
	DB $52,$21
cv3:
	; 		
	; 		' check controller 1 button 2 to start/reset game "A button"
	; 		if cont1.button2 then
	LD A,(joy1_data)
	AND 128
	JP Z,cv4
	; 			gosub init_vars ' initialize variables	' starting turret shot y position
	CALL cvb_INIT_VARS
	; 			'start_pressed = 1				
	; 		end if		
cv4:
	; 	end if	
cv1:
	; 	' --- end GAME OVER state 1 - game over ---
	; 		
	; 		
	; 		
	; 	
	; 	' --- GAME OVER state 0 - game running ---
	; 	if game_over = 0 then
	LD A,(cvb_GAME_OVER)
	AND A
	JP NZ,cv5
	; 			
	; 		
	; 		if inv_hit = 0 then 
	LD A,(cvb_INV_HIT)
	AND A
	JP NZ,cv6
	; 			gosub animate_move_inv	' animate and move invader
	CALL cvb_ANIMATE_MOVE_INV
	; 		end if
cv6:
	; 		
	; 		
	; 		' invader fire
	; 		inv_fire_delay = inv_fire_delay - 1	
	LD A,(cvb_INV_FIRE_DELAY)
	DEC A
	LD (cvb_INV_FIRE_DELAY),A
	; 		if inv_fire_delay = 0 then
	JP NZ,cv7
	; 			inv_fire_delay = 120
	LD A,120
	LD (cvb_INV_FIRE_DELAY),A
	; 			inv_fired = 1
	LD A,1
	LD (cvb_INV_FIRED),A
	; 			inv_shot_x = inv_x
	LD A,(cvb_INV_X)
	LD (cvb_INV_SHOT_X),A
	; 			inv_shot_y = inv_y + 6		
	LD A,(cvb_INV_Y)
	ADD A,6
	LD (cvb_INV_SHOT_Y),A
	; 		end if
cv7:
	; 				
	; 		
	; 		' check controller 1 left/right to move turret 
	; 		if cont1.left and tur_x >= 10 and tur_hit = 0 then 
	LD A,(cvb_TUR_X)
	CP 10
	LD A,0
	JR C,$+3
	DEC A
	LD B,A
	LD A,(joy1_data)
	AND 8
	AND B
	LD B,A
	LD A,(cvb_TUR_HIT)
	AND A
	LD A,0
	JR NZ,$+3
	DEC A
	AND B
	JP Z,cv8
	; 			tur_x = tur_x - 2
	LD A,(cvb_TUR_X)
	SUB 2
	LD (cvb_TUR_X),A
	; 		end if	
cv8:
	; 		if cont1.right and tur_x <= 234 and tur_hit = 0 then
	LD A,(cvb_TUR_X)
	CP 235
	LD A,0
	JR NC,$+3
	DEC A
	LD B,A
	LD A,(joy1_data)
	AND 2
	AND B
	LD B,A
	LD A,(cvb_TUR_HIT)
	AND A
	LD A,0
	JR NZ,$+3
	DEC A
	AND B
	JP Z,cv9
	; 			tur_x = tur_x + 2
	LD A,(cvb_TUR_X)
	ADD A,2
	LD (cvb_TUR_X),A
	; 		end if
cv9:
	; 			
	; 			
	; 		' check controller 1 button 1 to fire turret "B button"	
	; 		if cont1.button and fire_pressed = 0 and tur_hit = 0 and tur_fired = 0 then
	LD A,(cvb_FIRE_PRESSED)
	AND A
	LD A,0
	JR NZ,$+3
	DEC A
	LD B,A
	LD A,(joy1_data)
	AND 64
	AND B
	LD B,A
	LD A,(cvb_TUR_HIT)
	AND A
	LD A,0
	JR NZ,$+3
	DEC A
	AND B
	LD B,A
	LD A,(cvb_TUR_FIRED)
	AND A
	LD A,0
	JR NZ,$+3
	DEC A
	AND B
	JP Z,cv10
	; 			fire_pressed = 1
	LD A,1
	LD (cvb_FIRE_PRESSED),A
	; 			tur_fired = 1			
	LD A,1
	LD (cvb_TUR_FIRED),A
	; 			tur_shot_x = tur_x
	LD A,(cvb_TUR_X)
	LD (cvb_TUR_SHOT_X),A
	; 			tur_shot_y = tur_y				
	LD A,(cvb_TUR_Y)
	LD (cvb_TUR_SHOT_Y),A
	; 			'sound_tur_shot = 1			
	; 		end if
cv10:
	; 				
	; 		
	; 		' stops autofire by making sure button is released after press
	; 		if cont1.button = 0 and tur_fired = 0 then			
	LD A,(cvb_TUR_FIRED)
	AND A
	LD A,0
	JR NZ,$+3
	DEC A
	LD B,A
	LD A,(joy1_data)
	AND 64
	LD A,0
	JR NZ,$+3
	DEC A
	AND B
	JP Z,cv11
	; 			fire_pressed = 0			
	SUB A
	LD (cvb_FIRE_PRESSED),A
	; 		end if
cv11:
	; 				
	; 		
	; 		' check if turret shot collides with invader
	; 		t1 = tur_shot_x + 6		' turret shot's x (single pixel wide)
	LD A,(cvb_TUR_SHOT_X)
	ADD A,6
	LD (cvb_T1),A
	; 		t2 = tur_shot_y + 3		' turret shot's top edge
	LD A,(cvb_TUR_SHOT_Y)
	ADD A,3
	LD (cvb_T2),A
	; 		is1 = inv_x				' invader's left edge
	LD A,(cvb_INV_X)
	LD (cvb_IS1),A
	; 		is2 = inv_x + 10		' invader's right edge
	LD A,(cvb_INV_X)
	ADD A,10
	LD (cvb_IS2),A
	; 		is3 = inv_y + 15		' invader's bottom edge			
	LD A,(cvb_INV_Y)
	ADD A,15
	LD (cvb_IS3),A
	; 		' do actual check using variables from above				
	; 		if t1 > is1 and t1 < is2 and t2 < is3 then
	LD B,A
	LD A,(cvb_T2)
	CP B
	LD A,0
	JR NC,$+3
	DEC A
	PUSH AF
	LD A,(cvb_IS2)
	LD B,A
	LD A,(cvb_T1)
	CP B
	LD A,0
	JR NC,$+3
	DEC A
	PUSH AF
	LD A,(cvb_T1)
	LD B,A
	LD A,(cvb_IS1)
	CP B
	LD A,0
	JR NC,$+3
	DEC A
	POP BC
	AND B
	POP BC
	AND B
	JP Z,cv12
	; 			if inv_hit = 0 then
	LD A,(cvb_INV_HIT)
	AND A
	JP NZ,cv13
	; 				inv_hit = 1
	LD A,1
	LD (cvb_INV_HIT),A
	; 				#score = #score + 10
	LD HL,(cvb_#SCORE)
	LD DE,10
	ADD HL,DE
	LD (cvb_#SCORE),HL
	; 				'sound_inv_hit = 1				
	; 			end if
cv13:
	; 		end if
cv12:
	; 				
	; 		
	; 		' check if invader shot collides with turret
	; 		is1 = inv_shot_x + 4  	' invader shot's left edge 
	LD A,(cvb_INV_SHOT_X)
	ADD A,4
	LD (cvb_IS1),A
	; 		is2 = inv_shot_x + 6  	' invader shot's right edge
	LD A,(cvb_INV_SHOT_X)
	ADD A,6
	LD (cvb_IS2),A
	; 		is3 = inv_shot_y + 15  	' invader shot's bottom edge
	LD A,(cvb_INV_SHOT_Y)
	ADD A,15
	LD (cvb_IS3),A
	; 		t1 = tur_x				' turret's left edge
	LD A,(cvb_TUR_X)
	LD (cvb_T1),A
	; 		t2 = tur_x + 12 		' turret's right edge
	LD A,(cvb_TUR_X)
	ADD A,12
	LD (cvb_T2),A
	; 		t3 = tur_y + 11			' turret's top edge
	LD A,(cvb_TUR_Y)
	ADD A,11
	LD (cvb_T3),A
	; 		' do actual check using variables from above			
	; 		if is2 > t1 and is1 < t2 and is3 > t3 then
	LD A,(cvb_IS3)
	LD B,A
	LD A,(cvb_T3)
	CP B
	LD A,0
	JR NC,$+3
	DEC A
	PUSH AF
	LD A,(cvb_T2)
	LD B,A
	LD A,(cvb_IS1)
	CP B
	LD A,0
	JR NC,$+3
	DEC A
	PUSH AF
	LD A,(cvb_IS2)
	LD B,A
	LD A,(cvb_T1)
	CP B
	LD A,0
	JR NC,$+3
	DEC A
	POP BC
	AND B
	POP BC
	AND B
	JP Z,cv14
	; 			tur_hit = 1				
	LD A,1
	LD (cvb_TUR_HIT),A
	; 		end if
cv14:
	; 		
	; 		
	; 		
	; 		
	; 		' draw stuff below here -----------------------------------------------------
	; 		
	; 		
	; 		
	; 				
	; 		' draw turret sprites		
	; 		if tur_hit = 0 then 
	LD A,(cvb_TUR_HIT)
	AND A
	JP NZ,cv15
	; 			' if turret HAS NOT been hit draw regular turret
	; 			sprite 0,tur_y,tur_x,0,2
	SUB A
	PUSH AF
	LD A,(cvb_TUR_Y)
	PUSH AF
	LD A,(cvb_TUR_X)
	PUSH AF
	SUB A
	PUSH AF
	LD A,2
	CALL update_sprite
	; 			' hide turret explodion frames
	; 			sprite 1,190,255,4,2
	LD A,1
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,4
	PUSH AF
	LD A,2
	CALL update_sprite
	; 			sprite 2,190,255,8,2
	LD A,2
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,8
	PUSH AF
	LD A,2
	CALL update_sprite
	; 		' if turret HAS been hit play turret exploding animation
	; 		else
	JP cv16
cv15:
	; 			' hide turret sprite
	; 			sprite 0,190,255,0,2
	SUB A
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	SUB A
	PUSH AF
	LD A,2
	CALL update_sprite
	; 			' play turret exploding animation 3 times			
	; 			if tur_exp_times = 0 then 				
	LD A,(cvb_TUR_EXP_TIMES)
	AND A
	JP NZ,cv17
	; 				tur_hit = 0
	SUB A
	LD (cvb_TUR_HIT),A
	; 				tur_exp_times = 3 	' play animation 3 times
	LD A,3
	LD (cvb_TUR_EXP_TIMES),A
	; 				lives = lives - 1	' lose life				
	LD A,(cvb_LIVES)
	DEC A
	LD (cvb_LIVES),A
	; 				if lives = 0 then 	' if lives = 0...
	JP NZ,cv18
	; 					game_over = 1 	' ...set game state to game over								
	LD A,1
	LD (cvb_GAME_OVER),A
	; 				end if	
cv18:
	; 			end if
cv17:
	; 			
	; 			' keeps track of turret explosion frames (1 and 2)
	; 			tur_exp_delay = tur_exp_delay - 1
	LD A,(cvb_TUR_EXP_DELAY)
	DEC A
	LD (cvb_TUR_EXP_DELAY),A
	; 			if tur_exp_delay = 0 then 
	JP NZ,cv19
	; 				tur_exp_frame = tur_exp_frame + 1				
	LD A,(cvb_TUR_EXP_FRAME)
	INC A
	LD (cvb_TUR_EXP_FRAME),A
	; 				if tur_exp_frame > 2 then
	CP 3
	JP C,cv20
	; 					tur_exp_times = tur_exp_times - 1
	LD A,(cvb_TUR_EXP_TIMES)
	DEC A
	LD (cvb_TUR_EXP_TIMES),A
	; 					tur_exp_frame = 1					
	LD A,1
	LD (cvb_TUR_EXP_FRAME),A
	; 				end if			
cv20:
	; 				tur_exp_delay = 15
	LD A,15
	LD (cvb_TUR_EXP_DELAY),A
	; 			end if
cv19:
	; 			' draw turret exploding frames
	; 			if tur_exp_frame = 1 then
	LD A,(cvb_TUR_EXP_FRAME)
	CP 1
	JP NZ,cv21
	; 				' draw turret explodion frame 1
	; 				sprite 1,tur_y,tur_x,4,2
	LD A,1
	PUSH AF
	LD A,(cvb_TUR_Y)
	PUSH AF
	LD A,(cvb_TUR_X)
	PUSH AF
	LD A,4
	PUSH AF
	LD A,2
	CALL update_sprite
	; 				' hide turret explodion frame 2
	; 				sprite 2,190,255,8,2
	LD A,2
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,8
	PUSH AF
	LD A,2
	CALL update_sprite
	; 			end if	
cv21:
	; 			if tur_exp_frame = 2 then		
	LD A,(cvb_TUR_EXP_FRAME)
	CP 2
	JP NZ,cv22
	; 				' draw turret explodion frame 2
	; 				sprite 2,tur_y,tur_x,8,2
	LD A,2
	PUSH AF
	LD A,(cvb_TUR_Y)
	PUSH AF
	LD A,(cvb_TUR_X)
	PUSH AF
	LD A,8
	PUSH AF
	LD A,2
	CALL update_sprite
	; 				' hide turret explodion frame 1
	; 				sprite 1,190,255,4,2
	LD A,1
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,4
	PUSH AF
	LD A,2
	CALL update_sprite
	; 			end if
cv22:
	; 		end if
cv16:
	; 		
	; 		
	; 		' DEBUG TEXT
	; 		'print at 100,tur_hit
	; 		'print at 132,tur_exp_times
	; 		'print at 164,tur_exp_frame
	; 		
	; 		
	; 		' draw turret shot sprite	
	; 		if tur_fired = 1 then 					
	LD A,(cvb_TUR_FIRED)
	CP 1
	JP NZ,cv23
	; 			SPRITE 3,tur_shot_y,tur_shot_x,12,$f					
	LD A,3
	PUSH AF
	LD A,(cvb_TUR_SHOT_Y)
	PUSH AF
	LD A,(cvb_TUR_SHOT_X)
	PUSH AF
	LD A,12
	PUSH AF
	LD A,15
	CALL update_sprite
	; 			tur_shot_y = tur_shot_y - 4
	LD A,(cvb_TUR_SHOT_Y)
	SUB 4
	LD (cvb_TUR_SHOT_Y),A
	; 			if tur_shot_y < 10 then ' if shot goes off bottom of screen
	CP 10
	JP NC,cv24
	; 				tur_fired = 0
	SUB A
	LD (cvb_TUR_FIRED),A
	; 				tur_shot_x = 255
	LD A,255
	LD (cvb_TUR_SHOT_X),A
	; 				tur_shot_y = 190
	LD A,190
	LD (cvb_TUR_SHOT_Y),A
	; 				SPRITE 3,tur_shot_y,tur_shot_x,12,$f
	LD A,3
	PUSH AF
	LD A,(cvb_TUR_SHOT_Y)
	PUSH AF
	LD A,(cvb_TUR_SHOT_X)
	PUSH AF
	LD A,12
	PUSH AF
	LD A,15
	CALL update_sprite
	; 			end if		
cv24:
	; 		end if	
cv23:
	; 				
	; 				
	; 		' draw invader sprites
	; 		if inv_hit = 0 then
	LD A,(cvb_INV_HIT)
	AND A
	JP NZ,cv25
	; 			if inv_frame = 1 then
	LD A,(cvb_INV_FRAME)
	CP 1
	JP NZ,cv26
	; 				' invader frame 1		
	; 				sprite 4,inv_y,inv_x,16,$a
	LD A,4
	PUSH AF
	LD A,(cvb_INV_Y)
	PUSH AF
	LD A,(cvb_INV_X)
	PUSH AF
	LD A,16
	PUSH AF
	LD A,10
	CALL update_sprite
	; 				sprite 5,190,255,20,$a
	LD A,5
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,20
	PUSH AF
	LD A,10
	CALL update_sprite
	; 			elseif inv_frame = 2 then
	JP cv27
cv26:
	LD A,(cvb_INV_FRAME)
	CP 2
	JP NZ,cv28
	; 				' invader frame 2	
	; 				sprite 5,inv_y,inv_x,20,$a
	LD A,5
	PUSH AF
	LD A,(cvb_INV_Y)
	PUSH AF
	LD A,(cvb_INV_X)
	PUSH AF
	LD A,20
	PUSH AF
	LD A,10
	CALL update_sprite
	; 				sprite 4,190,255,16,$a			
	LD A,4
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,16
	PUSH AF
	LD A,10
	CALL update_sprite
	; 			end if
cv27:
cv28:
	; 		end if
cv25:
	; 		
	; 		' show invader blast
	; 		if inv_hit = 1 then
	LD A,(cvb_INV_HIT)
	CP 1
	JP NZ,cv29
	; 			' hide both invader sprites
	; 			sprite 4,190,255,16,$a
	LD A,4
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,16
	PUSH AF
	LD A,10
	CALL update_sprite
	; 			sprite 5,190,255,20,$a
	LD A,5
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,20
	PUSH AF
	LD A,10
	CALL update_sprite
	; 			' draw invader blast
	; 			sprite 6,inv_y,inv_x,24,$a
	LD A,6
	PUSH AF
	LD A,(cvb_INV_Y)
	PUSH AF
	LD A,(cvb_INV_X)
	PUSH AF
	LD A,24
	PUSH AF
	LD A,10
	CALL update_sprite
	; 		
	; 			inv_blast_delay = inv_blast_delay - 1
	LD A,(cvb_INV_BLAST_DELAY)
	DEC A
	LD (cvb_INV_BLAST_DELAY),A
	; 			if inv_blast_delay = 0 then ' reset all this stuff
	JP NZ,cv30
	; 				inv_hit = 0
	SUB A
	LD (cvb_INV_HIT),A
	; 				inv_fire_delay = 120
	LD A,120
	LD (cvb_INV_FIRE_DELAY),A
	; 				inv_blast_delay = 60
	LD A,60
	LD (cvb_INV_BLAST_DELAY),A
	; 				inv_delay = 30			
	LD A,30
	LD (cvb_INV_DELAY),A
	; 				inv_dir = 1
	LD A,1
	LD (cvb_INV_DIR),A
	; 				
	; 				' hide invader blast
	; 				sprite 6,190,255,24,$a
	LD A,6
	PUSH AF
	LD A,190
	PUSH AF
	LD A,255
	PUSH AF
	LD A,24
	PUSH AF
	LD A,10
	CALL update_sprite
	; 				
	; 				' set (fake) random x for invader 				
	; 				inv_x = r(array_pointer)
	LD HL,(cvb_ARRAY_POINTER)
	LD H,0
	LD DE,array_R
	ADD HL,DE
	LD A,(HL)
	LD (cvb_INV_X),A
	; 				array_pointer = array_pointer + 1
	LD A,(cvb_ARRAY_POINTER)
	INC A
	LD (cvb_ARRAY_POINTER),A
	; 				if array_pointer > 4 then
	CP 5
	JP C,cv31
	; 					array_pointer = 0
	SUB A
	LD (cvb_ARRAY_POINTER),A
	; 				end if
cv31:
	; 				inv_y = 8
	LD A,8
	LD (cvb_INV_Y),A
	; 			end if			
cv30:
	; 		end if		
cv29:
	; 		
	; 		
	; 		' draw invader shot
	; 		if inv_fired = 1 then			
	LD A,(cvb_INV_FIRED)
	CP 1
	JP NZ,cv32
	; 			sprite 7,inv_shot_y,inv_shot_x,28,$f	
	LD A,7
	PUSH AF
	LD A,(cvb_INV_SHOT_Y)
	PUSH AF
	LD A,(cvb_INV_SHOT_X)
	PUSH AF
	LD A,28
	PUSH AF
	LD A,15
	CALL update_sprite
	; 			inv_shot_y = inv_shot_y + 4
	LD A,(cvb_INV_SHOT_Y)
	ADD A,4
	LD (cvb_INV_SHOT_Y),A
	; 			
	; 			if inv_shot_y >= 168 then ' if shot goes off bottom of screen
	CP 168
	JP C,cv33
	; 				inv_fired = 0
	SUB A
	LD (cvb_INV_FIRED),A
	; 				inv_shot_x = 255
	LD A,255
	LD (cvb_INV_SHOT_X),A
	; 				inv_shot_y = 190
	LD A,190
	LD (cvb_INV_SHOT_Y),A
	; 				' hide invader shot sprite
	; 				sprite 7,inv_shot_y,inv_shot_x,28,$f
	LD A,7
	PUSH AF
	LD A,(cvb_INV_SHOT_Y)
	PUSH AF
	LD A,(cvb_INV_SHOT_X)
	PUSH AF
	LD A,28
	PUSH AF
	LD A,15
	CALL update_sprite
	; 			end if		
cv33:
	; 		end if	
cv32:
	; 		
	; 		
	; 		' print score and lives
	; 		print AT 0,"SCORE:",#score		
	LD HL,0
	LD (cursor),HL
	LD HL,cv34
	LD A,6
	CALL print_string
	JP cv35
cv34:
	DB $53,$43,$4f,$52,$45,$3a
cv35:
	LD HL,(cvb_#SCORE)
	CALL print_number
	; 		print AT 25,"LIVES:",lives
	LD HL,25
	LD (cursor),HL
	LD HL,cv36
	LD A,6
	CALL print_string
	JP cv37
cv36:
	DB $4c,$49,$56,$45,$53,$3a
cv37:
	LD HL,(cvb_LIVES)
	LD H,0
	CALL print_number
	; 	
	; 	
	; 	end if	
cv5:
	; 	' --- end GAME OVER state 0 - game running ---		
	; 	
	; 	
	; 	goto main_loop
	JP cvb_MAIN_LOOP
	; ' end main loop -------------------------------------------------------	
	; 	
	; 
	; ' animate and move invader
	; animate_move_inv:  procedure
cvb_ANIMATE_MOVE_INV:
	; 	inv_delay = inv_delay - 1	
	LD A,(cvb_INV_DELAY)
	DEC A
	LD (cvb_INV_DELAY),A
	; 	if inv_delay = 0 then
	JP NZ,cv38
	; 		inv_delay = 15		
	LD A,15
	LD (cvb_INV_DELAY),A
	; 					
	; 		' if dir = 1 then move right, if hit right reverse and drop	
	; 		if inv_dir = 1 then
	LD A,(cvb_INV_DIR)
	CP 1
	JP NZ,cv39
	; 			inv_x = inv_x + 2
	LD A,(cvb_INV_X)
	ADD A,2
	LD (cvb_INV_X),A
	; 				if inv_x >= 234 then 		
	CP 234
	JP C,cv40
	; 					inv_dir = 0
	SUB A
	LD (cvb_INV_DIR),A
	; 					inv_y = inv_y + 3					
	LD A,(cvb_INV_Y)
	ADD A,3
	LD (cvb_INV_Y),A
	; 				end if		
cv40:
	; 		' if dir = 0 then move left, if hit left reverse and drop
	; 		elseif inv_dir = 0 then
	JP cv41
cv39:
	LD A,(cvb_INV_DIR)
	AND A
	JP NZ,cv42
	; 			inv_x = inv_x - 2
	LD A,(cvb_INV_X)
	SUB 2
	LD (cvb_INV_X),A
	; 				if inv_x <= 10 then 		
	CP 11
	JP NC,cv43
	; 					inv_dir = 1
	LD A,1
	LD (cvb_INV_DIR),A
	; 					inv_y = inv_y + 3				
	LD A,(cvb_INV_Y)
	ADD A,3
	LD (cvb_INV_Y),A
	; 				end if
cv43:
	; 		end if		
cv41:
cv42:
	; 		
	; 		' update invader frame 1,2,1,2...
	; 		inv_frame = inv_frame + 1
	LD A,(cvb_INV_FRAME)
	INC A
	LD (cvb_INV_FRAME),A
	; 		if inv_frame > 2 then 
	CP 3
	JP C,cv44
	; 			inv_frame = 1
	LD A,1
	LD (cvb_INV_FRAME),A
	; 		end if		
cv44:
	; 	end if
cv38:
	; 
	; 	return
	RET
	; 	end	
	; 
	; 	
	; 		
	; 	' includes - should be at end		
	; 	include "globals.bas" 
	; ' globals.bas
	; 		
	; 
	; ' PROCEDURES
	; init_vars:  procedure
cvb_INIT_VARS:
	; 	cls
	CALL cls
	; 	
	; 	' turret variables	
	; 	tur_x = 128-6			' turret starting x position
	LD A,122
	LD (cvb_TUR_X),A
	; 	tur_y = 168				' turret starting y position
	LD A,168
	LD (cvb_TUR_Y),A
	; 	tur_fired = 0			' has turret fired? 0=false/1=true
	SUB A
	LD (cvb_TUR_FIRED),A
	; 	tur_shot_x = 255		' starting turret shot x position
	LD A,255
	LD (cvb_TUR_SHOT_X),A
	; 	tur_shot_y = 190		' starting turret shot y position
	LD A,190
	LD (cvb_TUR_SHOT_Y),A
	; 	tur_hit = 0				' has turret been hit? 0=false/1=true
	SUB A
	LD (cvb_TUR_HIT),A
	; 	tur_exp_frame = 1		' turret exploding starting frame
	LD A,1
	LD (cvb_TUR_EXP_FRAME),A
	; 	tur_exp_delay = 15		' turret exploding animation delay
	LD A,15
	LD (cvb_TUR_EXP_DELAY),A
	; 	tur_exp_times = 3		' number of times turret exploding animation plays
	LD A,3
	LD (cvb_TUR_EXP_TIMES),A
	; 	
	; 	' invader variables
	; 	inv_x = 128-5			' invader starting x position
	LD A,123
	LD (cvb_INV_X),A
	; 	inv_y = 8				' invader starting y position
	LD A,8
	LD (cvb_INV_Y),A
	; 	inv_shot_x = 255		' starting invader shot x position
	LD A,255
	LD (cvb_INV_SHOT_X),A
	; 	inv_shot_y = 190		' starting invader shot y position
	LD A,190
	LD (cvb_INV_SHOT_Y),A
	; 	inv_frame = 1			' invader starting frame
	LD A,1
	LD (cvb_INV_FRAME),A
	; 	inv_delay = 30			' invader move/animate delay
	LD A,30
	LD (cvb_INV_DELAY),A
	; 	inv_dir = 1				' starting invader direction, 0=left/1=right
	LD A,1
	LD (cvb_INV_DIR),A
	; 	inv_blast_delay	= 60	' how long to hold invader blast				
	LD A,60
	LD (cvb_INV_BLAST_DELAY),A
	; 	inv_fire_delay = 120	' delay between invader firing
	LD A,120
	LD (cvb_INV_FIRE_DELAY),A
	; 	inv_fired = 0			' has invader fired? 0=false/1=true
	SUB A
	LD (cvb_INV_FIRED),A
	; 	inv_shot_x = 0			' starting invader shot x position
	SUB A
	LD (cvb_INV_SHOT_X),A
	; 	inv_shot_y = 0			' starting invader shot y position
	SUB A
	LD (cvb_INV_SHOT_Y),A
	; 	inv_hit = 0				' has invader been hit?
	SUB A
	LD (cvb_INV_HIT),A
	; 	array_pointer = 0		' array pointer for fake random invader restart x position
	SUB A
	LD (cvb_ARRAY_POINTER),A
	; 
	; 	' sound variables
	; 	'sound_tur_shot = 0
	; 	'sound_inv_hit = 0
	; 	'sound_tur_hit = 0
	; 	
	; 	' misc variables
	; 	#score = 0				' starting score
	LD HL,0
	LD (cvb_#SCORE),HL
	; 	lives = 3				' starting live
	LD A,3
	LD (cvb_LIVES),A
	; 	fire_pressed = 0		' check if button is held/released	
	SUB A
	LD (cvb_FIRE_PRESSED),A
	; 	inv_hit = 0				' has invader been hit?	
	SUB A
	LD (cvb_INV_HIT),A
	; 	game_over = 0			' is game over?		
	SUB A
	LD (cvb_GAME_OVER),A
	; 	
	; 	return
	RET
	; 	end
	; 	
	; 	include "sprites.bas"
	; ' sprites.bas
	; 
	; 
	; sprites:
cvb_SPRITES:
	; ' turret: 
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "......X........."
	; 	BITMAP ".....XXX........"
	; 	BITMAP ".....XXX........"
	; 	BITMAP ".XXXXXXXXXXX...."
	; 	BITMAP "XXXXXXXXXXXXX..."
	; 	BITMAP "XXXXXXXXXXXXX..."
	; 	BITMAP "XXXXXXXXXXXXX..."
	; 	BITMAP "XXXXXXXXXXXXX..."
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $02,$07,$07,$7f,$ff,$ff,$ff,$ff
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $00,$00,$00,$f0,$f8,$f8,$f8,$f8
	; 	
	; ' turret explosion frame 1:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP ".....X.........."
	; 	BITMAP "..........X....."
	; 	BITMAP ".....X.X.X......"
	; 	BITMAP "..X..X.........."
	; 	BITMAP "......XX.XX....."
	; 	BITMAP "X..X..XX.X.X...."
	; 	BITMAP "..XXXXXXXX..X..."
	; 	BITMAP ".XXXXXXXXXX.X.X."
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $04,$00,$05,$24,$03,$93,$3f,$7f
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $00,$20,$40,$00,$60,$50,$c8,$ea
	; 
	; ' turret explosion frame 2:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "..X.........X..."
	; 	BITMAP "X....X....XX..X."
	; 	BITMAP "..X....XX......."
	; 	BITMAP ".....X.......X.."
	; 	BITMAP ".X.X.XX.XXX...X."
	; 	BITMAP "..X...XXX...X..."
	; 	BITMAP "...XXXXXXXX....."
	; 	BITMAP "..XX.XXXXXX..X.."
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $20,$84,$21,$04,$56,$23,$1f,$37
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $08,$32,$80,$04,$e2,$88,$e0,$e4
	; 
	; ' turret shot:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "......X........."
	; 	BITMAP "......X........."
	; 	BITMAP "......X........."
	; 	BITMAP "......X........."
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	DB $00,$00,$00,$02,$02,$02,$02,$00
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $00,$00,$00,$00,$00,$00,$00,$00
	; 	
	; ' invader frame 1:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "..X.....X......."
	; 	BITMAP "...X...X........"
	; 	BITMAP "..XXXXXXX......."
	; 	BITMAP ".XX.XXX.XX......"
	; 	BITMAP "XXXXXXXXXXX....."
	; 	BITMAP "XXXXXXXXXXX....."
	; 	BITMAP "X.X.....X.X....."
	; 	BITMAP "...X...X........"	
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $20,$11,$3f,$6e,$ff,$ff,$a0,$11
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $80,$00,$80,$c0,$e0,$e0,$a0,$00
	; 	
	; ' invader frame 2:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "..X.....X......."
	; 	BITMAP "X..X...X..X....."
	; 	BITMAP "X.XXXXXXX.X....."
	; 	BITMAP "XXX.XXX.XXX....."
	; 	BITMAP "XXXXXXXXXXX....."
	; 	BITMAP ".XXXXXXXXX......"
	; 	BITMAP "..X.....X......."
	; 	BITMAP ".X.......X......"
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $20,$91,$bf,$ee,$ff,$7f,$20,$40
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $80,$20,$a0,$e0,$e0,$c0,$80,$40
	; 	
	; 
	; ' invader blast:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "...X...X........"
	; 	BITMAP ".X..X.X..X......"
	; 	BITMAP "..X.....X......."
	; 	BITMAP "...X...X........"
	; 	BITMAP "XX.......XX....."
	; 	BITMAP "...X...X........"
	; 	BITMAP "..X.....X......."
	; 	BITMAP ".X..X.X..X......"
	; 	BITMAP "...X...X........"
	DB $00,$00,$00,$00,$00,$00,$00,$11
	DB $4a,$20,$11,$c0,$11,$20,$4a,$11
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $40,$80,$00,$60,$00,$80,$40,$00
	; 
	; ' invader shot:
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP "................"
	; 	BITMAP ".....X.........."
	; 	BITMAP "......X........."
	; 	BITMAP ".....X.........."
	; 	BITMAP "....X..........."
	; 	BITMAP ".....X.........."	
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $00,$00,$00,$04,$02,$04,$08,$04
	DB $00,$00,$00,$00,$00,$00,$00,$00
	DB $00,$00,$00,$00,$00,$00,$00,$00
	; 	
	; 	
	; 
	; 	
	;
	; CVBasic epilogue (BASIC compiler for Colecovision)
	;
	; by Oscar Toledo G.
	; https://nanochess.org/
	;
	; Creation date: Feb/27/2024.
	; Revision date: Feb/29/2024. Added joystick, keypad, frame, random, and
	;                             read_pointer variables.
	; Revision date: Mar/04/2024. Added music player.
	; Revision date: Mar/05/2024. Added support for Sega SG1000.
	; Revision date: Mar/12/2024. Added support for MSX.
	; Revision date: Mar/13/2024. Added Pletter decompressor.
	; Revision date: Mar/19/2024. Added support for sprite flicker.
	; Revision date: Apr/11/2024. Added support for Super Game Module.
	; Revision date: Apr/13/2024. Updates LFSR in interruption handler.
	; Revision date: Apr/26/2024. All code moved to cvbasic_prologue.asm so it
	;                             can remain accessible in bank 0 (bank switching).
	;

	org BASE_RAM

sprites:
	rb 128
sprite_data:
	rb 4
frame:
	rb 2
read_pointer:
	rb 2
cursor:
	rb 2
lfsr:
	rb 2
mode:
	rb 1
flicker:
	rb 1
joy1_data:
	rb 1
joy2_data:
	rb 1
key1_data:
	rb 1
key2_data:
	rb 1
ntsc:
	rb 1

    if CVBASIC_MUSIC_PLAYER
music_tick:             rb 1
music_mode:             rb 1

    if CVBASIC_BANK_SWITCHING
music_bank:             rb 1
    endif
music_start:		rb 2
music_pointer:		rb 2
music_playing:		rb 1
music_timing:		rb 1
music_note_counter:	rb 1
music_instrument_1:	rb 1
music_counter_1:	rb 1
music_note_1:		rb 1
music_instrument_2:	rb 1
music_counter_2:	rb 1
music_note_2:		rb 1
music_instrument_3:	rb 1
music_counter_3:	rb 1
music_note_3:		rb 1
music_counter_4:	rb 1
music_drum:		rb 1

audio_freq1:		rb 2
audio_freq2:		rb 2
audio_freq3:		rb 2
audio_noise:		rb 1
audio_mix:		rb 1
audio_vol1:		rb 1
audio_vol2:		rb 1
audio_vol3:		rb 1

audio_control:		rb 1
audio_vol4hw:		rb 1
    endif

    if SGM
	org $2000	; Start for variables.
    endif
cvb_LIVES:	rb 1
cvb_INV_HIT:	rb 1
cvb_ARRAY_POINTER:	rb 1
cvb_INV_SHOT_X:	rb 1
cvb_INV_SHOT_Y:	rb 1
cvb_TUR_EXP_DELAY:	rb 1
cvb_#SCORE:	rb 2
cvb_TUR_SHOT_X:	rb 1
cvb_TUR_SHOT_Y:	rb 1
cvb_INV_X:	rb 1
cvb_INV_Y:	rb 1
cvb_INV_DELAY:	rb 1
cvb_TUR_X:	rb 1
cvb_TUR_Y:	rb 1
cvb_TUR_EXP_FRAME:	rb 1
cvb_INV_FIRED:	rb 1
cvb_TUR_FIRED:	rb 1
cvb_TUR_HIT:	rb 1
cvb_INV_FRAME:	rb 1
cvb_GAME_OVER:	rb 1
cvb_INV_BLAST_DELAY:	rb 1
cvb_INV_DIR:	rb 1
cvb_TUR_EXP_TIMES:	rb 1
cvb_IS1:	rb 1
cvb_IS2:	rb 1
cvb_IS3:	rb 1
cvb_T1:	rb 1
cvb_T2:	rb 1
cvb_T3:	rb 1
cvb_FIRE_PRESSED:	rb 1
cvb_INV_FIRE_DELAY:	rb 1
array_R:	rb 5
